# $Id: Configuration.pm 29027 2009-10-17 12:30:28Z wsl $
# $URL: https://infix.uvt.nl/its-id/trunk/sources/aselect-perl/lib/Aselect/Configuration.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package Aselect::Configuration;

use Xyzzy::Configuration -Base;

use Aselect::Static;
use Aselect::Status;
use Aselect::Server;
use Aselect::Login;
use Aselect::LDAP;

field 'url';
field 'id';
field 'organization' => 'ACME';
field 'randomdev';
field 'timeout' => 3600;
field 'attributes' => {};
field 'ldap_server';
field 'ldap_base';
field 'ldap_filter';
field 'ldap_attribute';

*set_aselecturl = *url;
*set_aselectserverid = *id;
*set_aselectorganization = *organization;
*set_aselectldapserver = *ldap_server;
*set_aselectldapbase = *ldap_base;
*set_aselectldapfilter = *ldap_filter;
*set_aselectldapattribute = *ldap_attribute;

sub set_aselectattribute {
	my $value = $_[0];
	die "AselectAttribute requires at least one argument\n"
		unless defined $value;
	my $attributes = $self->attributes;
	$self->attributes($attributes = {})
		unless %$attributes;
	my ($key, $val) = split(' ', $value, 2);
	$attributes->{$key} = $val || $key;
}

our %units = (
	'' => 1,
	s => 1,
	m => 60,
	h => 3600,
	d => 86400,
	w => 604800,
	m => 31 * 86400,
	q => 92 * 86400,
	y => int(364.2425 * 86400),
);

sub set_sessiontimeout {
	my $time = $_[0];
	$time =~ /^(\d+)\s*([a-z]?)$/i;
	my ($seconds, $unit) = ($1, $2);
	if(defined $seconds) {
		$unit = lc($unit);
		if(exists $units{$unit}) {
			$unit = $units{$unit};
		} else {
			warn "unknown SessionTimeout unit $1, assuming seconds\n";
			$unit = 1;
		}
		$self->timeout($seconds * $unit);
	} else {
		warn "can't parse SessionTimeout $time, assuming one hour\n";
		$self->timeout(3600);
	}
}

sub set_entropydevice {
	my $path = $_[0];
	warn "EntropyDevice $path: No such file or directory\n"
		unless -e $path;
	$self->randomdev($path);
}

sub fallback { 'Aselect::Static' }

our %generators = (
	'' => 'Aselect::Status',
	server => 'Aselect::Server',
	login => 'Aselect::LDAP',
);

sub generators { \%generators }
